import json
import re
from pathlib import Path

from playwright.sync_api import expect

from umap.models import DataLayer, Map

from ..base import DataLayerFactory


def intercept_remote_data(page):
    data = [
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {"name": "Point 2", "foobar": "bla"},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [4.3375, 11.2707],
                    },
                }
            ],
        },
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {"name": "Point 1", "foobar": "baz"},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [4.3375, 12.2707],
                    },
                }
            ],
        },
    ]

    def handle(route):
        route.fulfill(json=data.pop())

    # Intercept the route to the proxy
    page.route("https://remote.org/data.json", handle)
    page.on("request", lambda *a, **k: print(a, k))


def test_dynamic_remote_data(page, live_server, tilelayer, map):
    intercept_remote_data(page)
    settings = {
        "remoteData": {
            "url": "https://remote.org/data.json",
            "format": "geojson",
            "dynamic": True,
        },
        "showLabel": True,
    }
    DataLayerFactory(map=map, settings=settings)
    map.edit_status = Map.ANONYMOUS
    map.settings["properties"]["zoom"] = 6
    map.settings["geometry"] = {
        "type": "Point",
        "coordinates": [5, 12],
    }
    map.save()

    page.goto(f"{live_server.url}{map.get_absolute_url()}")

    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()

    # Now drag the map
    map_el = page.locator("#map")
    map_el.drag_to(
        map_el,
        source_position={"x": 100, "y": 100},
        target_position={"x": 110, "y": 110},
    )

    expect(page.get_by_role("tooltip", name="Point 2")).to_be_visible()
    # Needed otherwise it found two (!) tooltip with name "Point 1"…
    page.wait_for_timeout(300)
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_hidden()

    # Map must not be dirty
    page.get_by_role("button", name="Edit").click()
    expect(page.locator(".edit-undo")).to_be_disabled()


def test_create_remote_data_layer(page, live_server, tilelayer, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    intercept_remote_data(page)
    page.goto(f"{live_server.url}/en/map/new#6/11.2707/4.3375")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Add a layer").click()
    page.get_by_text("Remote data").click()
    page.locator('.panel input[name="url"]').fill("https://remote.org/data.json")
    # We have a setTimeout on each input to throttle, so wait for it
    page.wait_for_timeout(300)
    page.locator('select[name="format"]').select_option("geojson")
    # with page.expect_response(re.compile("https://remote.org/data.json")):
    page.get_by_role("button", name="Verify remote URL").click()
    expect(page.locator(".leaflet-marker-icon")).to_have_count(1)
    with page.expect_response(re.compile(".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save draft", exact=True).click()
    datalayer = DataLayer.objects.last()
    data = json.loads(Path(datalayer.geojson.path).read_text())
    assert data == {
        "_umap_options": {
            "browsable": True,
            "displayOnLoad": True,
            "editMode": "advanced",
            "fields": [
                {
                    "key": "name",
                    "type": "String",
                },
                {
                    "key": "foobar",
                    "type": "String",
                },
            ],
            "id": str(datalayer.pk),
            "inCaption": True,
            "name": "Layer 1",
            "rank": 0,
            "remoteData": {
                "format": "geojson",
                "url": "https://remote.org/data.json",
            },
        },
        "features": [],
        "type": "FeatureCollection",
    }



from playwright.sync_api import expect


def test_reverse_search(live_server, page, tilelayer):
    photon_response = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {
                    "osm_type": "N",
                    "osm_id": 5055853416,
                    "osm_key": "place",
                    "osm_value": "locality",
                    "type": "locality",
                    "postcode": "10200",
                    "countrycode": "FR",
                    "name": "Le Haut Sentier",
                    "country": "France",
                    "city": "Thors",
                    "state": "Grand Est",
                    "county": "Aube",
                },
                "geometry": {"type": "Point", "coordinates": [4.7995256, 48.2985251]},
            }
        ],
    }
    page.goto(f"{live_server.url}/en/map/new")

    def handle_search(route):
        route.fulfill(json=photon_response)

    # Intercept the route
    page.route(
        "https://photon.komoot.io/reverse/?limit=1&lat=48.3&lon=4.8",
        handle_search,
    )
    page.get_by_role("button", name="Search location").click()
    page.get_by_role("searchbox", name="Type a place name or").fill("48.3 4.8")
    expect(page.get_by_text("48.3 4.8")).to_be_visible()
    expect(page.get_by_text("Le Haut Sentier")).to_be_visible()



import stat
from pathlib import Path

import pytest

from umap.utils import gzip_file, normalize_string


def test_gzip_file(settings):
    settings.FILE_UPLOAD_PERMISSIONS = 0o666
    # Let's use any old file so we can check that the date of the gzip file is set.
    src = Path(__file__).parent / "settings.py"
    dest = Path("/tmp/test_settings.py.gz")
    gzip_file(src, dest)
    src_stat = src.stat()
    dest_stat = dest.stat()
    dest.unlink()
    assert src_stat.st_mtime == dest_stat.st_mtime
    assert stat.filemode(dest_stat.st_mode) == "-rw-rw-rw-"


@pytest.mark.parametrize(
    "input,output",
    (
        ("Vélo", "velo"),
        ("Éducation", "education"),
        ("stävänger", "stavanger"),
    ),
)
def test_normalize_string(input, output):
    assert normalize_string(input) == output



import json
import re
from copy import deepcopy
from pathlib import Path

import pytest
from playwright.sync_api import expect

from umap.models import DataLayer, Map

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


DATALAYER_DATA1 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 2",
                "mynumber": 10,
                "mydate": "2024/04/14 12:19:17",
            },
            "geometry": {"type": "Point", "coordinates": [0.065918, 48.385442]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 1",
                "mynumber": 12,
                "mydate": "2024/03/13 12:20:20",
            },
            "geometry": {"type": "Point", "coordinates": [3.55957, 49.767074]},
        },
    ],
    "_umap_options": {
        "name": "Calque 1",
    },
}


DATALAYER_DATA2 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 4",
                "mynumber": 10,
                "mydate": "2024/08/18 13:14:15",
            },
            "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 3",
                "mynumber": 14,
                "mydate": "2024-04-14T10:19:17.000Z",
            },
            "geometry": {"type": "Point", "coordinates": [4.372559, 47.945786]},
        },
    ],
    "_umap_options": {
        "name": "Calque 2",
    },
}


def test_can_add_field_on_map(live_server, page, openmap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Add a new field").click()
    page.get_by_role("textbox", name="Field Name ✔").fill("newfield")
    page.get_by_label("Field Type").select_option("Number")
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/update/settings/")):
        page.get_by_role("button", name="Save").click()
    saved = Map.objects.get(pk=openmap.pk)
    assert saved.settings["properties"]["fields"] == [
        {"key": "newfield", "type": "Number"}
    ]


def test_can_add_field_on_datalayer(live_server, page, openmap, datalayer):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Add a new field").click()
    page.get_by_role("textbox", name="Field Name ✔").fill("newfield")
    page.get_by_label("Field Type").select_option("Number")
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {"key": "name", "type": "String"},
        {"key": "description", "type": "Text"},
        {"key": "newfield", "type": "Number"},
    ]


def test_edit_and_rename_field_from_datalayer(live_server, page, openmap):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Edit this field").first.click()
    page.get_by_role("textbox", name="Field Name ✔").fill("mytypenew")
    page.get_by_label("Field Type").select_option("Text")
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {
            "key": "mytypenew",
            "type": "Text",
        },
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mynumber",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "mytypenew": "odd",
        "name": "Point 1",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "mytypenew": "even",
        "name": "Point 2",
    }
    page.locator(".edit-undo").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {
            "key": "mytype",
            "type": "String",
        },
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mynumber",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "mytype": "odd",
        "name": "Point 1",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "mytype": "even",
        "name": "Point 2",
    }


def test_edit_and_rename_field_from_map(live_server, page, openmap):
    dl1 = DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    DataLayerFactory(map=openmap, data=DATALAYER_DATA2)
    openmap.settings["properties"]["fields"] = [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
    ]
    openmap.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Edit this field").first.click()
    page.get_by_role("textbox", name="Field Name ✔").fill("mytypenew")
    page.get_by_label("Field Type").select_option("Text")
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = Map.objects.get(pk=openmap.pk)
    assert saved.settings["properties"]["fields"] == [
        {"key": "mytypenew", "type": "Text"},
        {"key": "mynumber", "type": "Number"},
    ]
    saved = DataLayer.objects.get(pk=dl1.pk)
    assert saved.settings["fields"] == [
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "mytypenew": "odd",
        "name": "Point 1",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "mytypenew": "even",
        "name": "Point 2",
    }
    page.locator(".edit-undo").click()

    with page.expect_response(re.compile(rf".*/datalayer/update/{dl1.pk}")):
        with page.expect_response(re.compile(r".*/update/settings/")):
            page.get_by_role("button", name="Save").click()
    saved = Map.objects.get(pk=openmap.pk)
    assert saved.settings["properties"]["fields"] == [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
    ]
    saved = DataLayer.objects.get(pk=dl1.pk)
    assert saved.settings["fields"] == [
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "mytype": "odd",
        "name": "Point 1",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "mytype": "even",
        "name": "Point 2",
    }


def test_delete_field_from_datalayer(live_server, page, openmap):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Delete this field").first.click()
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mynumber",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "name": "Point 1",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "name": "Point 2",
    }
    page.locator(".edit-undo").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {
            "key": "mytype",
            "type": "String",
        },
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mynumber",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "name": "Point 1",
        "mytype": "odd",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "name": "Point 2",
        "mytype": "even",
    }


def test_delete_field_from_map(live_server, page, openmap):
    dl1 = DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    DataLayerFactory(map=openmap, data=DATALAYER_DATA2)
    openmap.settings["properties"]["fields"] = [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
    ]
    openmap.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Manage Fields").click()
    # Delete field mytype
    page.get_by_role("button", name="Delete this field").first.click()
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = Map.objects.get(pk=openmap.pk)
    assert saved.settings["properties"]["fields"] == [
        {"key": "mynumber", "type": "Number"},
    ]
    saved = DataLayer.objects.get(pk=dl1.pk)
    assert saved.settings["fields"] == [
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "name": "Point 1",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "name": "Point 2",
    }
    page.locator(".edit-undo").click()
    with page.expect_response(re.compile(r"./update/settings/.*")):
        with page.expect_response(re.compile(rf".*/datalayer/update/{dl1.pk}/")):
            page.get_by_role("button", name="Save").click()
    saved = Map.objects.get(pk=openmap.pk)
    assert saved.settings["properties"]["fields"] == [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
    ]
    saved = DataLayer.objects.get(pk=dl1.pk)
    assert saved.settings["fields"] == [
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "name": "Point 1",
        "mytype": "odd",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "name": "Point 2",
        "mytype": "even",
    }


def test_delete_field_from_datalayer_also_in_map(live_server, page, openmap):
    # mytype exist both on map and datalayer 1
    # deleting the field in the datalayer should not delete the data, as the field
    # is also defined on the map
    data = deepcopy(DATALAYER_DATA1)
    data["_umap_options"]["fields"] = [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
        {"key": "name", "type": "String"},
        {"key": "mydate", "type": "Date"},
    ]
    DataLayerFactory(map=openmap, data=data)
    openmap.settings["properties"]["fields"] = [
        {"key": "mytype", "type": "String"},
    ]
    openmap.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).first.click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Delete this field").first.click()
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {"key": "mynumber", "type": "Number"},
        {"key": "name", "type": "String"},
        {"key": "mydate", "type": "Date"},
    ]
    data = json.loads(Path(saved.geojson.path).read_text())
    assert data["features"][0]["properties"] == {
        "mydate": "2024/03/13 12:20:20",
        "mynumber": 12,
        "name": "Point 1",
        "mytype": "odd",
    }
    assert data["features"][1]["properties"] == {
        "mydate": "2024/04/14 12:19:17",
        "mynumber": 10,
        "name": "Point 2",
        "mytype": "even",
    }


def test_can_change_field_type_with_remote_data(live_server, page, openmap, tilelayer):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"name": "Point 2", "myenum": "foofoo,bababar"},
                "geometry": {
                    "type": "Point",
                    "coordinates": [4.3375, 11.2707],
                },
            },
            {
                "type": "Feature",
                "properties": {"name": "Point 1", "myenum": "feefee,bababar"},
                "geometry": {
                    "type": "Point",
                    "coordinates": [4.3375, 12.2707],
                },
            },
        ],
    }

    def handle(route):
        route.fulfill(json=data)

    DataLayerFactory(
        map=openmap,
        settings={
            "remoteData": {
                "url": "https://remote.org/data.json",
                "format": "geojson",
            },
        },
    )
    # Intercept the route to the proxy
    page.route("https://remote.org/data.json", handle)

    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#9/12.0017/4.4824")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.locator("summary").filter(has_text="Manage Fields").click()
    # Click on "myenum" edit button
    page.get_by_role("button", name="Edit this field").nth(1).click()
    page.get_by_label("Field Type").select_option("Enum")
    page.get_by_role("button", name="OK").click()
    # Click on "myenum" add filter button
    page.get_by_role("button", name="Add a filter for this field").nth(1).click()
    page.get_by_role("button", name="OK").click()
    page.get_by_role("button", name="Open browser").click()
    page.get_by_text("Filters", exact=True).click()
    expect(page.locator(".panel .umap-filter label")).to_contain_text(
        ["bababar", "feefee", "foofoo"]
    )


def test_boolean_field_should_display_a_switch_in_feature_form(
    live_server, page, openmap, tilelayer
):
    openmap.settings["properties"]["fields"] = [
        {"key": "mystring", "type": "String"},
        {"key": "mynumber", "type": "Number"},
        {"key": "mybool", "type": "Boolean"},
    ]
    openmap.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Draw a marker (Ctrl+M)").click()
    page.locator("#map").click()
    panel = page.locator(".panel")
    expect(panel.locator(".umap-field-mynumber input")).to_have_attribute(
        "type", "number"
    )
    expect(panel.locator(".umap-field-mybool.with-switch")).to_be_visible()



import copy
import re

import pytest
from playwright.sync_api import expect

from umap.models import DataLayer, Map

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


DATALAYER_DATA1 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 2",
                "mynumber": 10,
                "mydate": "2024/04/14 12:19:17",
            },
            "geometry": {"type": "Point", "coordinates": [0.065918, 48.385442]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 1",
                "mynumber": 12,
                "mydate": "2024/03/13 12:20:20",
            },
            "geometry": {"type": "Point", "coordinates": [3.55957, 49.767074]},
        },
    ],
    "_umap_options": {
        "name": "Calque 1",
    },
}


DATALAYER_DATA2 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 4",
                "mynumber": 10,
                "mydate": "2024/08/18 13:14:15",
            },
            "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 3",
                "mynumber": 14,
                "mydate": "2024-04-14T10:19:17.000Z",
            },
            "geometry": {"type": "Point", "coordinates": [4.372559, 47.945786]},
        },
    ],
    "_umap_options": {
        "name": "Calque 2",
    },
}


DATALAYER_DATA3 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {"name": "a polygon"},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [2.12, 49.57],
                        [1.08, 49.02],
                        [2.51, 47.55],
                        [3.19, 48.77],
                        [2.12, 49.57],
                    ]
                ],
            },
        },
    ],
    "_umap_options": {"name": "Calque 2", "browsable": False},
}


def test_simple_facet_search(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["filters"] = [
        {"fieldKey": "mytype", "label": "My type"},
        {"fieldKey": "mynumber", "label": "My number", "widget": "MinMax"},
    ]
    map.settings["properties"]["fields"] = [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
    ]
    map.settings["properties"]["showLabel"] = True
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    DataLayerFactory(map=map, data=DATALAYER_DATA3)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    panel = page.locator(".panel.left.on")
    expect(panel).to_have_class(re.compile(".*expanded.*"))
    expect(panel.locator(".umap-browser")).to_be_visible()
    # From a non browsable datalayer, should not be impacted
    paths = page.locator(".leaflet-overlay-pane path")
    expect(paths).to_be_visible()
    expect(panel).to_be_visible()
    # Facet name
    expect(page.get_by_text("My type")).to_be_visible()
    # Facet values
    oven = page.get_by_text("even")
    odd = page.get_by_text("odd")
    expect(oven).to_be_visible()
    expect(odd).to_be_visible()
    expect(paths).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)
    # Tooltips
    # Sometimes PW founds two tooltips with the same name, but cannot reproduce it.
    page.wait_for_timeout(300)
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 2")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 3")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 4")).to_be_visible()

    # Datalist
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 2")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()
    expect(panel.get_by_text("Point 4")).to_be_visible()

    # Now let's filter
    odd.click()
    expect(markers).to_have_count(2)
    expect(page.get_by_role("tooltip", name="Point 2")).to_be_hidden()
    expect(page.get_by_role("tooltip", name="Point 4")).to_be_hidden()
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 3")).to_be_visible()
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()
    expect(paths).to_be_visible
    # Now let's filter
    odd.click()
    expect(markers).to_have_count(4)
    expect(paths).to_be_visible()

    # Let's filter using the number facet
    expect(page.get_by_text("My Number")).to_be_visible()
    expect(page.get_by_label("Min")).to_have_value("10")
    expect(page.get_by_label("Max")).to_have_value("14")
    page.get_by_label("Min").fill("11")
    page.keyboard.press("Tab")  # Move out of the input, so the "change" event is sent
    expect(markers).to_have_count(2)
    expect(paths).to_be_visible()
    page.get_by_label("Max").fill("13")
    page.keyboard.press("Tab")
    expect(markers).to_have_count(1)

    # Now let's combine
    page.get_by_label("Min").fill("10")
    page.keyboard.press("Tab")
    expect(markers).to_have_count(3)
    odd.click()
    expect(markers).to_have_count(1)
    expect(paths).to_be_visible()


def test_date_facet_search(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["filters"] = [
        {"fieldKey": "mydate", "label": "Date filter", "widget": "MinMax"}
    ]
    map.settings["properties"]["fields"] = [{"key": "mydate", "type": "Date"}]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/-1.5")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)
    expect(page.get_by_text("Date Filter")).to_be_visible()
    expect(page.get_by_label("From")).to_have_value("2024-03-13")
    expect(page.get_by_label("Until")).to_have_value("2024-08-18")
    page.get_by_label("From").fill("2024-03-14")
    expect(markers).to_have_count(3)
    page.get_by_label("Until").fill("2024-08-17")
    expect(markers).to_have_count(2)


def test_choice_with_empty_value(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["fields"] = [{"key": "mytype", "type": "String"}]
    map.settings["properties"]["filters"] = [{"fieldKey": "mytype", "label": "My type"}]
    map.save()
    data = copy.deepcopy(DATALAYER_DATA1)
    data["features"][0]["properties"]["mytype"] = ""
    del data["features"][1]["properties"]["mytype"]
    DataLayerFactory(map=map, data=data)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/-1.5")
    expect(page.get_by_text("<empty value>")).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)
    page.get_by_text("<empty value>").click()
    expect(markers).to_have_count(2)


def test_number_with_zero_value(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["filters"] = [
        {"fieldKey": "mynumber", "label": "Filter", "widget": "MinMax"}
    ]
    map.settings["properties"]["fields"] = [{"key": "mynumber", "type": "Number"}]
    map.save()
    data = copy.deepcopy(DATALAYER_DATA1)
    data["features"][0]["properties"]["mynumber"] = 0
    DataLayerFactory(map=map, data=data)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/-1.5")
    expect(page.get_by_label("Min")).to_have_value("0")
    expect(page.get_by_label("Max")).to_have_value("14")
    page.get_by_label("Min").fill("1")
    page.keyboard.press("Tab")  # Move out of the input, so the "change" event is sent
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(3)


def test_facets_search_are_persistent_when_closing_panel(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["filters"] = [
        {"fieldKey": "mytype", "label": "My type"},
        {"fieldKey": "mynumber", "label": "My Number", "widget": "MinMax"},
    ]
    map.settings["properties"]["fields"] = [
        {"key": "mytype"},
        {"key": "mynumber", "type": "Number"},
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    panel = page.locator(".panel.left")

    # Facet values
    odd = page.get_by_label("odd")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)

    # Datalist in the browser
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 2")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()
    expect(panel.get_by_text("Point 4")).to_be_visible()

    # Now let's filter
    odd.click()
    expect(page.locator(".filters summary")).to_have_attribute("data-badge", " ")
    expect(page.locator(".umap-control-browse")).to_have_attribute("data-badge", " ")
    expect(markers).to_have_count(2)
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()

    # Let's filter using the number facet
    expect(panel.get_by_label("Min")).to_have_value("10")
    expect(panel.get_by_label("Max")).to_have_value("14")
    page.get_by_label("Min").fill("13")
    page.keyboard.press("Tab")  # Move out of the input, so the "change" event is sent
    expect(panel.get_by_label("Min")).to_have_attribute("data-modified", "true")
    expect(markers).to_have_count(1)
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_hidden()
    expect(panel.get_by_text("Point 3")).to_be_visible()

    # Close panel
    expect(panel.locator(".filters summary")).to_have_attribute("data-badge", " ")
    expect(page.locator(".umap-control-browse")).to_have_attribute("data-badge", " ")
    panel.get_by_role("button", name="Close").click()
    page.get_by_role("button", name="Open browser").click()
    expect(panel.get_by_label("Min")).to_have_value("13")
    expect(panel.get_by_label("Min")).to_have_attribute("data-modified", "true")
    expect(panel.get_by_label("odd")).to_be_checked()

    # Datalist in the browser should be inchanged
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_hidden()
    expect(panel.get_by_text("Point 3")).to_be_visible()


def test_can_load_legacy_facetKey(live_server, page, openmap):
    openmap.settings["properties"]["facetKey"] = (
        "mytype|My Type|radio,mynumber|My Number|number,mydate|My Date|date"
    )
    openmap.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    expect(
        page.get_by_text("The map has been upgraded to latest version, please save it.")
    ).to_be_visible()
    with page.expect_response(re.compile("./update/settings/.*")):
        page.get_by_role("button", name="Save", exact=True).click()
    saved = Map.objects.first()
    assert "facetKey" not in saved.settings["properties"]
    assert saved.settings["properties"]["filters"] == [
        {
            "fieldKey": "mytype",
            "label": "My Type",
            "widget": "Radio",
        },
        {
            "fieldKey": "mynumber",
            "label": "My Number",
            "widget": "MinMax",
        },
        {
            "fieldKey": "mydate",
            "label": "My Date",
            "widget": "MinMax",
        },
    ]
    assert saved.settings["properties"]["fields"] == [
        {"key": "mytype", "type": "String"},
        {"key": "mynumber", "type": "Number"},
        {"key": "mydate", "type": "Date"},
    ]


def test_deleting_field_should_delete_filter(live_server, page, openmap, datalayer):
    datalayer.settings["fields"] = [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    datalayer.settings["filters"] = [
        {"fieldKey": "foobar", "widget": "MinMax", "label": "Foo Bar"},
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]
    datalayer.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Delete this field").nth(1).click()
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {"key": "name", "type": "String"},
        {"key": "description", "type": "Text"},
    ]
    saved.settings["filters"] == [
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]
    page.locator(".edit-undo").click()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    saved.settings["filters"] == [
        {"fieldKey": "foobar", "widget": "MinMax", "label": "Foo Bar"},
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]


def test_deleting_field_from_map_should_delete_filter(live_server, page, openmap):
    openmap.settings["properties"]["fields"] = [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    openmap.settings["properties"]["filters"] = [
        {"fieldKey": "foobar", "widget": "MinMax", "label": "Foo Bar"},
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]
    openmap.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Manage Fields").click()
    page.get_by_role("button", name="Delete this field").nth(1).click()
    page.get_by_role("button", name="OK").click()
    with page.expect_response(re.compile(r"./update/settings/.*")):
        page.get_by_role("button", name="Save").click()
    saved = Map.objects.first()
    assert saved.settings["properties"]["fields"] == [
        {"key": "name", "type": "String"},
        {"key": "description", "type": "Text"},
    ]
    saved.settings["properties"]["filters"] == [
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]
    page.locator(".edit-undo").click()
    with page.expect_response(re.compile(r"./update/settings/.*")):
        page.get_by_role("button", name="Save").click()
    saved = Map.objects.first()
    assert saved.settings["properties"]["fields"] == [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    saved.settings["properties"]["filters"] == [
        {"fieldKey": "foobar", "widget": "MinMax", "label": "Foo Bar"},
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]


def test_can_create_filter_from_new_field(live_server, page, openmap):
    openmap.settings["properties"]["onLoadPanel"] = "datafilters"
    openmap.save()
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).nth(1).click()
    page.get_by_role("heading", name="Manage Fields").click()
    page.get_by_role("button", name="Add a new field").click()
    page.get_by_role("textbox", name="Field Name ✔").fill("foobar")
    page.get_by_label("Field Type").select_option("Boolean")
    page.get_by_role("button", name="Add filter for this field").click()
    page.get_by_role("textbox", name="Human readable name of the").fill("Foo Bar")
    expect(page.get_by_text("Yes/No")).to_be_checked()
    page.wait_for_timeout(300)  # Input throttling.
    page.get_by_text("Edit this field").click()
    expect(page.locator(".umap-filter span").filter(has_text="Foo Bar")).to_be_visible()
    with page.expect_response(re.compile(r".*/datalayer/update/")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.first()
    assert saved.settings["fields"] == [
        {
            "key": "mytype",
            "type": "String",
        },
        {
            "key": "name",
            "type": "String",
        },
        {
            "key": "mynumber",
            "type": "String",
        },
        {
            "key": "mydate",
            "type": "String",
        },
        {
            "key": "foobar",
            "type": "Boolean",
        },
    ]

    assert saved.settings["filters"] == [
        {
            "fieldKey": "foobar",
            "label": "Foo Bar",
            "widget": "Switch",
        },
    ]


def test_can_create_new_filter_on_map_from_panel(live_server, page, openmap):
    openmap.settings["properties"]["onLoadPanel"] = "datafilters"
    openmap.settings["properties"]["fields"] = [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    openmap.save()
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage filters").click()
    page.get_by_role("button", name="Add filter").first.click()
    page.get_by_label("Filter on").select_option("foobar")
    page.get_by_role("textbox", name="Human readable name").fill("Foo Bar")
    page.wait_for_timeout(300)
    page.get_by_text("Exclusive choice").click()
    page.get_by_role("button", name="OK").click()
    expect(
        page.locator(".panel.left").get_by_role("group", name="Foo Bar")
    ).to_be_visible()
    with page.expect_response(re.compile("./update/settings/.*")):
        page.get_by_role("button", name="Save", exact=True).click()
    saved = Map.objects.first()
    assert saved.settings["properties"]["filters"] == [
        {"fieldKey": "foobar", "label": "Foo Bar", "widget": "Radio"}
    ]


def test_can_create_new_filter_on_datalayer_from_panel(live_server, page, openmap):
    openmap.settings["properties"]["fields"] = [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    openmap.save()
    datalayer = DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    page.goto(
        f"{live_server.url}{openmap.get_absolute_url()}?edit&onLoadPanel=datafilters"
    )
    page.get_by_role("button", name="Manage filters").click()
    page.get_by_role("button", name="Add filter").nth(1).click()
    expect(page.get_by_label("Apply filter to")).to_have_value(f"layer:{datalayer.pk}")
    page.get_by_label("Filter on").select_option("mynumber")
    page.get_by_role("textbox", name="Human readable name of the").fill("Foo Bar")
    page.wait_for_timeout(300)
    page.get_by_text("Exclusive choice").click()
    page.get_by_role("button", name="OK").click()
    expect(
        page.locator(".panel.left").get_by_role("group", name="Foo Bar")
    ).to_be_visible()
    with page.expect_response(re.compile("./datalayer/update/.*")):
        page.get_by_role("button", name="Save", exact=True).click()
    saved = DataLayer.objects.first()
    assert saved.settings["filters"] == [
        {"fieldKey": "mynumber", "label": "Foo Bar", "widget": "Radio"}
    ]


def test_can_edit_filter_from_panel(live_server, page, openmap):
    openmap.settings["properties"]["fields"] = [
        {"key": "name", "type": "String"},
        {"key": "foobar", "type": "Number"},
        {"key": "description", "type": "Text"},
    ]
    openmap.settings["properties"]["filters"] = [
        {"fieldKey": "foobar", "widget": "MinMax", "label": "Foo Bar"},
        {"fieldKey": "name", "widget": "Checkbox", "label": "Bar Foo"},
    ]
    openmap.save()
    page.goto(
        f"{live_server.url}{openmap.get_absolute_url()}?edit&onLoadPanel=datafilters"
    )
    page.get_by_role("group", name="Foo Bar").get_by_role("button").click()
    expect(page.get_by_label("Apply filter to")).to_have_value(f"map:{openmap.pk}")
    expect(page.get_by_label("Apply filter to")).to_be_disabled()
    page.get_by_role("textbox", name="Human readable name of the").fill("Foo Bar Baz")
    page.wait_for_timeout(300)  # Input throttling.
    page.get_by_role("button", name="OK").click()
    expect(
        page.get_by_role("group", name="Foo Bar Baz").locator("legend")
    ).to_be_visible()


def test_filter_with_enum(live_server, openmap, page):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {
                    "name": "Point 2",
                    "products": "flour,vegetable,bread",
                },
                "geometry": {"type": "Point", "coordinates": [0.065918, 48.385442]},
            },
            {
                "type": "Feature",
                "properties": {
                    "name": "Point 1",
                    "products": "vegetable,bread",
                },
                "geometry": {"type": "Point", "coordinates": [3.55957, 49.767074]},
            },
            {
                "type": "Feature",
                "properties": {
                    "name": "Point 3",
                    "products": "flour,vegetable,milk,eggs",
                },
                "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
            },
        ],
        "_umap_options": {
            "name": "Calque 1",
            "fields": [{"key": "products", "type": "Enum"}],
        },
    }
    DataLayerFactory(map=openmap, data=data)
    page.goto(
        f"{live_server.url}{openmap.get_absolute_url()}?edit&onLoadPanel=datafilters#6/48.948/1.670"
    )
    page.get_by_role("button", name="Manage filters").click()
    page.get_by_role("button", name="Add filter").click()
    page.get_by_label("Filter on").select_option("products")
    page.get_by_role("button", name="OK").click()
    expect(
        page.locator(".umap-browser .filters span").filter(has_text="products")
    ).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(3)
    page.get_by_role("checkbox", name="bread").check()
    expect(markers).to_have_count(2)
    page.get_by_role("checkbox", name="bread").uncheck()
    expect(markers).to_have_count(3)
    page.get_by_role("checkbox", name="vegetable").check()
    expect(markers).to_have_count(3)
    page.get_by_role("checkbox", name="milk").check()
    expect(markers).to_have_count(1)



import pytest
from playwright.sync_api import expect

from .helpers import save_and_get_json

pytestmark = pytest.mark.django_db


def test_draw_polygon(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    # Click on the Draw a polygon button on a new map.
    create_path = page.locator(".umap-edit-bar").get_by_title("Draw a polygon")
    create_path.click()

    # Check no polygon is present by default.
    # We target with the color, because there is also the drawing line guide (dash-array)
    # around
    lines = page.locator(".leaflet-overlay-pane path[stroke='DarkBlue']")
    guide = page.locator(".leaflet-overlay-pane > svg > g > path")
    expect(lines).to_have_count(0)
    expect(guide).to_have_count(0)

    # Click on the map, it will create a polygon.
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 200})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(1)
    map.click(position={"x": 100, "y": 200})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(2)
    map.click(position={"x": 100, "y": 100})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(2)
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(0)


def test_clicking_esc_should_finish_polygon(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    # Click on the Draw a polygon button on a new map.
    create_path = page.locator(".umap-edit-bar ").get_by_title("Draw a polygon")
    create_path.click()

    # Check no polygon is present by default.
    # We target with the color, because there is also the drawing line guide (dash-array)
    # around
    lines = page.locator(".leaflet-overlay-pane path[stroke='DarkBlue']")
    guide = page.locator(".leaflet-overlay-pane > svg > g > path")
    expect(lines).to_have_count(0)
    expect(guide).to_have_count(0)

    # Click on the map, it will create a polygon.
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 200})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(1)
    map.click(position={"x": 100, "y": 200})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(2)
    map.click(position={"x": 100, "y": 100})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(2)
    # Click ESC to finish
    page.keyboard.press("Escape")
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(0)
    # Should have opened edit form panel
    expect(page.locator(".panel").get_by_text("Feature properties")).to_be_visible()


def test_clicking_esc_should_delete_polygon_if_empty(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    # Click on the Draw a polygon button on a new map.
    create_path = page.locator(".umap-edit-bar ").get_by_title("Draw a polygon")
    create_path.click()

    # Check no polygon is present by default.
    # We target with the color, because there is also the drawing line guide (dash-array)
    # around
    lines = page.locator(".leaflet-overlay-pane path[stroke='DarkBlue']")
    guide = page.locator(".leaflet-overlay-pane > svg > g > path")
    expect(lines).to_have_count(0)
    expect(guide).to_have_count(0)

    # Click ESC to finish, no polygon should have been created
    page.keyboard.press("Escape")
    expect(lines).to_have_count(0)
    expect(guide).to_have_count(0)
    # Should not have opened edit form panel
    expect(page.locator(".panel").get_by_text("Feature properties")).to_be_hidden()


def test_clicking_esc_should_delete_polygon_if_invalid(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    # Click on the Draw a polygon button on a new map.
    create_path = page.locator(".umap-edit-bar ").get_by_title("Draw a polygon")
    create_path.click()

    # Check no polygon is present by default.
    # We target with the color, because there is also the drawing line guide (dash-array)
    # around
    lines = page.locator(".leaflet-overlay-pane path[stroke='DarkBlue']")
    guide = page.locator(".leaflet-overlay-pane > svg > g > path")
    expect(lines).to_have_count(0)
    expect(guide).to_have_count(0)

    # Click on the map twice, it will start a polygon.
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 200})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(1)
    map.click(position={"x": 100, "y": 200})
    expect(lines).to_have_count(1)
    expect(guide).to_have_count(2)
    # Click ESC to finish, the polygon is invalid, it should not be persisted
    page.keyboard.press("Escape")
    expect(lines).to_have_count(0)
    expect(guide).to_have_count(0)
    # Should not have opened edit form panel
    expect(page.locator(".panel").get_by_text("Feature properties")).to_be_hidden()


def test_can_draw_multi(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")
    polygons = page.locator(".leaflet-overlay-pane path")
    expect(polygons).to_have_count(0)
    multi_button = page.get_by_title("Add a polygon to the current multi")
    expect(multi_button).to_be_hidden()
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 150, "y": 100})
    map.click(position={"x": 150, "y": 150})
    map.click(position={"x": 100, "y": 150})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(multi_button).to_be_visible()
    expect(polygons).to_have_count(1)
    multi_button.click()
    map.click(position={"x": 250, "y": 200})
    map.click(position={"x": 250, "y": 250})
    map.click(position={"x": 200, "y": 250})
    map.click(position={"x": 200, "y": 200})
    # Click again to finish
    map.click(position={"x": 200, "y": 200})
    expect(polygons).to_have_count(1)
    page.keyboard.press("Escape")
    expect(multi_button).to_be_hidden()
    polygons.first.click(button="right", position={"x": 10, "y": 10})
    expect(page.get_by_role("button", name="Transform to lines")).to_be_hidden()
    expect(page.get_by_role("button", name="Delete this shape")).to_be_visible()


def test_can_draw_hole(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    page.get_by_title("Draw a polygon").click()

    polygons = page.locator(".leaflet-overlay-pane path")
    vertices = page.locator(".leaflet-vertex-icon")

    # Click on the map, it will create a polygon.
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 100})
    map.click(position={"x": 200, "y": 200})
    map.click(position={"x": 100, "y": 200})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(polygons).to_have_count(1)
    expect(vertices).to_have_count(4)

    page.wait_for_timeout(300)  # Time for the panel animation to finish
    # First vertex of the hole will be created here
    map.click(position={"x": 180, "y": 120}, button="right")
    page.get_by_role("button", name="Start a hole here").click()
    map.click(position={"x": 180, "y": 180})
    map.click(position={"x": 120, "y": 180})
    map.click(position={"x": 120, "y": 120})
    # Click again to finish
    map.click(position={"x": 120, "y": 120})
    expect(polygons).to_have_count(1)
    expect(vertices).to_have_count(8)
    # Click on the polygon but not in the hole
    polygons.first.click(button="right", position={"x": 10, "y": 10})
    expect(page.get_by_role("button", name="Transform to lines")).to_be_hidden()


def test_can_transfer_shape_from_simple_polygon(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")
    polygons = page.locator(".leaflet-overlay-pane path")
    expect(polygons).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")

    # Draw a first polygon
    map.click(position={"x": 150, "y": 100})
    map.click(position={"x": 150, "y": 150})
    map.click(position={"x": 100, "y": 150})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(polygons).to_have_count(1)

    # Draw another polygon
    page.get_by_title("Draw a polygon").click()
    map.click(position={"x": 250, "y": 200})
    map.click(position={"x": 250, "y": 250})
    map.click(position={"x": 200, "y": 250})
    map.click(position={"x": 200, "y": 200})
    # Click again to finish
    map.click(position={"x": 200, "y": 200})
    expect(polygons).to_have_count(2)

    # Now that polygon 2 is selected, right click on first one
    # and transfer shape
    polygons.first.click(position={"x": 20, "y": 20}, button="right")
    page.get_by_role("button", name="Transfer shape to edited feature").click()
    expect(polygons).to_have_count(1)


def test_can_extract_shape(live_server, page, tilelayer, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/en/map/new/")
    polygons = page.locator(".leaflet-overlay-pane path")
    expect(polygons).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 150, "y": 100})
    map.click(position={"x": 150, "y": 150})
    map.click(position={"x": 100, "y": 150})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(polygons).to_have_count(1)
    extract_button = page.get_by_role(
        "button", name="Extract shape to separate feature"
    )
    expect(extract_button).to_be_hidden()
    page.get_by_title("Add a polygon to the current multi").click()
    map.click(position={"x": 250, "y": 200})
    map.click(position={"x": 250, "y": 250})
    map.click(position={"x": 200, "y": 250})
    map.click(position={"x": 200, "y": 200})
    # Click again to finish
    map.click(position={"x": 200, "y": 200})
    expect(polygons).to_have_count(1)
    polygons.first.click(position={"x": 20, "y": 20}, button="right")
    extract_button.click()
    expect(polygons).to_have_count(2)
    data = save_and_get_json(page)
    assert len(data["features"]) == 2
    assert data["features"][0]["geometry"]["type"] == "Polygon"
    assert data["features"][1]["geometry"]["type"] == "Polygon"
    assert data["features"][0]["geometry"]["coordinates"] == [
        [
            [
                -6.569824,
                53.159947,
            ],
            [
                -6.569824,
                52.49616,
            ],
            [
                -7.668457,
                52.49616,
            ],
            [
                -7.668457,
                53.159947,
            ],
            [
                -6.569824,
                53.159947,
            ],
        ],
    ]
    assert data["features"][1]["geometry"]["coordinates"] == [
        [
            [
                -8.76709,
                54.457267,
            ],
            [
                -8.76709,
                53.813626,
            ],
            [
                -9.865723,
                53.813626,
            ],
            [
                -9.865723,
                54.457267,
            ],
            [
                -8.76709,
                54.457267,
            ],
        ],
    ]


def test_cannot_transfer_shape_to_line(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")
    polygons = page.locator(".leaflet-overlay-pane path")
    expect(polygons).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 150, "y": 100})
    map.click(position={"x": 150, "y": 150})
    map.click(position={"x": 100, "y": 150})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(polygons).to_have_count(1)
    extract_button = page.get_by_role(
        "button", name="Extract shape to separate feature"
    )
    polygons.first.click(position={"x": 20, "y": 20}, button="right")
    expect(extract_button).to_be_hidden()
    page.get_by_title("Draw a polyline").click()
    map.click(position={"x": 200, "y": 250})
    map.click(position={"x": 200, "y": 200})
    # Click again to finish
    map.click(position={"x": 200, "y": 200})
    expect(polygons).to_have_count(2)
    polygons.first.click(position={"x": 20, "y": 20}, button="right")
    expect(extract_button).to_be_hidden()


def test_cannot_transfer_shape_to_marker(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")
    polygons = page.locator(".leaflet-overlay-pane path")
    expect(polygons).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 150, "y": 100})
    map.click(position={"x": 150, "y": 150})
    map.click(position={"x": 100, "y": 150})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(polygons).to_have_count(1)
    extract_button = page.get_by_role(
        "button", name="Extract shape to separate feature"
    )
    polygons.first.click(position={"x": 20, "y": 20}, button="right")
    expect(extract_button).to_be_hidden()
    page.get_by_title("Draw a marker").click()
    map.click(position={"x": 250, "y": 200})
    expect(polygons).to_have_count(1)
    polygons.first.click(position={"x": 20, "y": 20}, button="right")
    expect(extract_button).to_be_hidden()


def test_can_clone_polygon(live_server, page, tilelayer, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/en/map/new/")
    polygons = page.locator(".leaflet-overlay-pane path")
    expect(polygons).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 100})
    map.click(position={"x": 200, "y": 200})
    map.click(position={"x": 100, "y": 200})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    page.wait_for_timeout(300)  # Time for the panel animation to finish
    expect(polygons).to_have_count(1)
    polygons.first.click(button="right")
    page.get_by_role("button", name="Clone this feature").click()
    expect(polygons).to_have_count(2)
    data = save_and_get_json(page)
    assert len(data["features"]) == 2
    assert data["features"][0]["geometry"]["type"] == "Polygon"
    assert data["features"][0]["geometry"] == data["features"][1]["geometry"]


def test_can_transform_polygon_to_line(live_server, page, tilelayer, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/en/map/new/")
    paths = page.locator(".leaflet-overlay-pane path")
    polygons = page.locator(".leaflet-overlay-pane path[fill='DarkBlue']")
    expect(polygons).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 100})
    map.click(position={"x": 200, "y": 200})
    map.click(position={"x": 100, "y": 200})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(polygons).to_have_count(1)
    expect(paths).to_have_count(1)
    page.wait_for_timeout(300)  # Time for the panel animation to finish
    polygons.first.click(button="right")
    page.get_by_role("button", name="Transform to lines").click()
    # No more polygons (will fill), but one path, it must be a line
    expect(polygons).to_have_count(0)
    expect(paths).to_have_count(1)
    data = save_and_get_json(page)
    assert len(data["features"]) == 1
    assert data["features"][0]["geometry"]["type"] == "LineString"


def test_can_draw_a_polygon_and_invert_it(live_server, page, tilelayer, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/en/map/new/")
    paths = page.locator(".leaflet-overlay-pane path")
    expect(paths).to_have_count(0)
    page.get_by_title("Draw a polygon").click()
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 100})
    map.click(position={"x": 200, "y": 200})
    map.click(position={"x": 100, "y": 200})
    map.click(position={"x": 100, "y": 100})
    # Click again to finish
    map.click(position={"x": 100, "y": 100})
    expect(paths).to_have_count(1)
    page.get_by_text("Advanced properties").click()
    page.get_by_text("Display the polygon inverted").click()
    data = save_and_get_json(page)
    # Close save message
    page.get_by_role("dialog").get_by_role("button", name="Close").click()
    assert len(data["features"]) == 1
    assert data["features"][0]["geometry"]["type"] == "Polygon"
    assert data["features"][0]["geometry"]["coordinates"] == [
        [
            [
                -7.668457,
                54.457267,
            ],
            [
                -7.668457,
                53.159947,
            ],
            [
                -9.865723,
                53.159947,
            ],
            [
                -9.865723,
                54.457267,
            ],
            [
                -7.668457,
                54.457267,
            ],
        ],
    ]

    page.get_by_role("button", name="View").click()
    popup = page.locator(".leaflet-popup")
    expect(popup).to_be_hidden()
    # Now click on the middle of the polygon, it should not show the popup
    map.click(position={"x": 150, "y": 150})
    expect(popup).to_be_hidden()
    # Click elsewhere on the map, it should now show the popup
    map.click(position={"x": 250, "y": 250})
    expect(popup).to_be_visible()


def test_vertexmarker_not_shown_if_too_many(live_server, map, page, settings):
    geojson = '{"type":"Feature","geometry":{"type":"Polygon","coordinates":[[[3.350602,48.438077],[3.349287,48.438082],[3.34921,48.438124],[3.348519,48.438108],[3.34546,48.437416],[3.343752,48.436955],[3.339092,48.435705],[3.333756,48.434278],[3.330224,48.433336],[3.326293,48.43229],[3.323154,48.430374],[3.32129,48.429238],[3.321234,48.429191],[3.321164,48.429221],[3.320893,48.429117],[3.320766,48.42912],[3.320575,48.429213],[3.320289,48.429303],[3.320042,48.429427],[3.319659,48.429542],[3.319215,48.429622],[3.318547,48.429691],[3.317845,48.429671],[3.317751,48.429698],[3.316503,48.430404],[3.316247,48.430481],[3.316101,48.431152],[3.316181,48.431164],[3.315466,48.432852],[3.315229,48.432981],[3.314785,48.433076],[3.314588,48.432699],[3.314474,48.432376],[3.314197,48.431965],[3.313812,48.431626],[3.313264,48.431253],[3.312393,48.430865],[3.311687,48.43069],[3.311471,48.430693],[3.311199,48.430622],[3.310632,48.430628],[3.30879,48.430373],[3.307032,48.430298],[3.306597,48.430211],[3.306301,48.430213],[3.306137,48.430161],[3.305651,48.430165],[3.304839,48.430046],[3.303726,48.429803],[3.302861,48.42972],[3.302237,48.429635],[3.300559,48.429488],[3.300396,48.429435],[3.299502,48.429335],[3.298528,48.429198],[3.298176,48.429201],[3.296263,48.429039],[3.296267,48.429307],[3.296237,48.429425],[3.295882,48.429848],[3.295665,48.429789],[3.295397,48.430056],[3.295377,48.430132],[3.295186,48.430421],[3.295198,48.430531],[3.295344,48.430735],[3.296077,48.431333],[3.295938,48.431617],[3.29576,48.43168],[3.294082,48.431442],[3.292288,48.431198],[3.292303,48.431101],[3.29082,48.431007],[3.29043,48.430975],[3.290451,48.431129],[3.290115,48.431105],[3.289097,48.430993],[3.289185,48.430805],[3.288545,48.430699],[3.288311,48.430684],[3.287686,48.430687],[3.287456,48.431129],[3.287465,48.43122],[3.288277,48.431574],[3.28896,48.431915],[3.288937,48.431969],[3.289431,48.432499],[3.289672,48.43292],[3.289871,48.433156],[3.29036,48.433602],[3.290557,48.433724],[3.290781,48.433809],[3.291035,48.433857],[3.291537,48.434024],[3.291819,48.434151],[3.292118,48.434341],[3.292479,48.434677],[3.292929,48.435388],[3.293207,48.435792],[3.293881,48.43672],[3.293762,48.436772],[3.294056,48.437209],[3.294117,48.437385],[3.294618,48.437579],[3.294465,48.437764],[3.294424,48.438087],[3.294357,48.438293],[3.293776,48.438817],[3.293308,48.439323],[3.292929,48.439844],[3.292671,48.440235],[3.29233,48.440924],[3.291807,48.441432],[3.29161,48.441661],[3.291402,48.44196],[3.291265,48.442663],[3.291255,48.442806],[3.291328,48.443126],[3.291407,48.443202],[3.291574,48.443473],[3.292253,48.444495],[3.292329,48.444596],[3.293056,48.445276],[3.293138,48.445309],[3.293368,48.445628],[3.293661,48.445985],[3.29374,48.446117],[3.29396,48.446372],[3.294304,48.446627],[3.294761,48.446912],[3.295881,48.447668],[3.295849,48.447688],[3.296837,48.448338],[3.297547,48.44891],[3.297465,48.44892],[3.297188,48.449195],[3.297597,48.449543],[3.297753,48.449701],[3.297845,48.449851],[3.298264,48.450055],[3.298478,48.450121],[3.298946,48.450221],[3.299309,48.450317],[3.299359,48.450237],[3.300493,48.450461],[3.301087,48.450674],[3.301703,48.45101],[3.301995,48.451197],[3.3024,48.451534],[3.302702,48.45174],[3.303329,48.452007],[3.304029,48.452197],[3.304569,48.452446],[3.304803,48.452502],[3.305096,48.452877],[3.30567,48.453409],[3.305998,48.453617],[3.306329,48.453567],[3.306999,48.453359],[3.307147,48.453453],[3.307452,48.453162],[3.307621,48.452853],[3.307637,48.452428],[3.307707,48.452345],[3.307741,48.452152],[3.307605,48.451823],[3.307551,48.45153],[3.307474,48.451395],[3.307218,48.451316],[3.307069,48.45119],[3.307261,48.450528],[3.307483,48.449868],[3.307603,48.449365],[3.30774,48.448909],[3.307598,48.448808],[3.307761,48.448604],[3.307863,48.447956],[3.307886,48.447645],[3.307972,48.447245],[3.308239,48.446362],[3.308306,48.446042],[3.308487,48.445329],[3.308442,48.444844],[3.308479,48.444713],[3.308967,48.443542],[3.309235,48.442927],[3.309464,48.442289],[3.309372,48.442046],[3.309621,48.441616],[3.310152,48.441065],[3.310213,48.440729],[3.310237,48.440329],[3.310167,48.439906],[3.31076,48.439111],[3.31118,48.438009],[3.311161,48.437961],[3.311906,48.437902],[3.312261,48.437839],[3.312486,48.437744],[3.31306,48.437674],[3.312613,48.438361],[3.312487,48.43883],[3.312493,48.439136],[3.312443,48.439388],[3.312598,48.440393],[3.312739,48.440752],[3.312879,48.440985],[3.313263,48.441305],[3.313916,48.441515],[3.314457,48.441565],[3.315105,48.44156],[3.31581,48.441607],[3.317056,48.441849],[3.318361,48.442198],[3.319041,48.442408],[3.319287,48.442604],[3.319343,48.442711],[3.320216,48.443117],[3.320709,48.443437],[3.32126,48.444007],[3.321788,48.444776],[3.322181,48.445618],[3.322479,48.445616],[3.32283,48.445577],[3.323344,48.445663],[3.324048,48.445693],[3.324695,48.445562],[3.324992,48.445559],[3.325558,48.445482],[3.325963,48.445479],[3.327479,48.445592],[3.327939,48.445678],[3.328502,48.445481],[3.328942,48.445392],[3.329169,48.44538],[3.330112,48.445466],[3.330715,48.445575],[3.330881,48.44557],[3.332155,48.445373],[3.33243,48.445375],[3.332727,48.445438],[3.3332,48.445588],[3.333358,48.445683],[3.333737,48.446027],[3.333998,48.446169],[3.334135,48.446334],[3.334611,48.447294],[3.33488,48.447909],[3.334992,48.447959],[3.335297,48.448013],[3.336516,48.448161],[3.336874,48.44825],[3.337258,48.448531],[3.337442,48.448737],[3.337525,48.448936],[3.337649,48.448967],[3.338263,48.448902],[3.33836,48.44894],[3.338765,48.44921],[3.339281,48.449513],[3.339464,48.449515],[3.339877,48.448856],[3.339867,48.448673],[3.340611,48.447311],[3.341744,48.447535],[3.343846,48.447943],[3.345266,48.448152],[3.345478,48.447345],[3.345816,48.446774],[3.345976,48.446809],[3.346142,48.44657],[3.346,48.446493],[3.346043,48.446099],[3.346047,48.445835],[3.346203,48.44558],[3.34717,48.444977],[3.347471,48.444638],[3.347571,48.444466],[3.347583,48.444183],[3.347678,48.443989],[3.348162,48.443428],[3.348326,48.443259],[3.348351,48.443136],[3.34831,48.442736],[3.348141,48.442484],[3.348246,48.442411],[3.348271,48.442293],[3.348097,48.442202],[3.347875,48.442142],[3.347773,48.441997],[3.34751,48.441531],[3.347394,48.441212],[3.349317,48.441364],[3.349478,48.441055],[3.349528,48.44103],[3.350119,48.441039],[3.350252,48.440793],[3.35052,48.440779],[3.350618,48.440612],[3.35069,48.440129],[3.350806,48.43921],[3.350792,48.439037],[3.350698,48.438594],[3.350566,48.438327],[3.350602,48.438077]]]},"properties":{"nom":"Grisy-sur-Seine","code":"77218","codeDepartement":"77","siren":"217702182","codeEpci":"200040251","codeRegion":"11","codesPostaux":["77480"],"population":107},"id":"g0OTg"}'
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/en/map/new/#15/48.4395/3.3189")
    page.get_by_title("Import data").click()
    page.wait_for_timeout(300)  # Time for the panel animation to finish
    page.locator(".umap-import textarea").fill(geojson)
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    page.locator("path").click(button="right")
    page.get_by_role("button", name="Toggle edit mode (⇧+Click)").click()
    expect(page.locator(".umap-tooltip-container")).to_contain_text(
        "Please zoom in to edit the geometry"
    )
    expect(page.locator(".leaflet-vertex-icon")).to_be_hidden()
    page.get_by_label("Zoom in").click()
    expect(page.locator(".umap-tooltip-container")).to_contain_text(
        "Please zoom in to edit the geometry"
    )
    page.get_by_label("Zoom in").click()
    page.wait_for_timeout(500)
    page.get_by_label("Zoom out").click()
    page.wait_for_timeout(500)
    expect(page.locator(".leaflet-vertex-icon")).to_be_hidden()
    expect(page.locator(".umap-tooltip-container")).to_contain_text(
        "Please zoom in to edit the geometry"
    )



import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


DATALAYER_DATA = {
    "type": "FeatureCollection",
    "_umap_options": {
        "name": "Calque 1",
        "type": "Cluster",
        "cluster": {},
        "browsable": True,
        "inCaption": True,
        "displayOnLoad": True,
    },
    "features": [
        {
            "type": "Feature",
            "properties": {"name": "one point in france"},
            "geometry": {"type": "Point", "coordinates": [3.339844, 46.920255]},
        },
        {
            "type": "Feature",
            "properties": {
                "name": "one another point in france in same position",
                "description": "can you see me ?",
            },
            "geometry": {"type": "Point", "coordinates": [3.339844, 46.920255]},
        },
        {
            "type": "Feature",
            "properties": {
                "name": "again one another point",
                "description": "and me ?",
            },
            "geometry": {"type": "Point", "coordinates": [3.34, 46.1]},
        },
    ],
}


def test_can_open_feature_on_browser_click(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#7/46.920/3.340")
    page.get_by_text("one another point in france in same position").click()
    expect(page.get_by_text("can you see me ?")).to_be_visible()
    page.get_by_text("again one another point").click()
    expect(page.get_by_text("and me ?")).to_be_visible()


def test_can_drag_single_marker_in_cluster_layer(live_server, page, tilelayer, openmap):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#7/46.920/3.340")

    marker = page.locator(".umap-div-icon")
    map = page.locator("#map")

    expect(page.locator(".edit-undo")).to_be_disabled()
    # Drag marker
    old_bbox = marker.bounding_box()
    marker.first.drag_to(map, target_position={"x": 250, "y": 250})
    assert marker.bounding_box() != old_bbox
    expect(page.locator(".edit-undo")).to_be_enabled()
    # Make sure edit stays panel
    page.wait_for_timeout(1000)
    expect(page.locator(".panel.right")).to_be_visible()


def test_can_drag_marker_in_cluster(live_server, page, tilelayer, openmap):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#18/46.92/3.34")

    marker = page.locator(".umap-div-icon")
    cluster = page.locator(".umap-cluster-icon")
    map = page.locator("#map")
    expect(marker).to_have_count(0)

    expect(page.locator(".edit-undo")).to_be_disabled()
    cluster.click()
    marker.first.drag_to(map, target_position={"x": 250, "y": 250})
    expect(page.locator(".edit-undo")).to_be_enabled()
    # There is no more cluster
    expect(marker).to_have_count(2)


def test_can_change_datalayer_of_marker_in_cluster(
    live_server, page, datalayer, openmap, tilelayer
):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA)
    datalayer.settings["iconClass"] = "Ball"
    datalayer.save()
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#7/46.920/3.340")

    expect(page.locator(".umap-ball-icon")).to_have_count(0)
    page.locator(".umap-div-icon").click(modifiers=["Shift"])
    page.get_by_role("combobox").select_option(str(datalayer.pk))
    expect(page.locator(".umap-ball-icon")).to_have_count(1)


def test_can_combine_cluster_with_remote_data_and_fromZoom(
    page, live_server, tilelayer, map
):
    settings = {
        "fromZoom": "7",
        "type": "Cluster",
        "showLabel": True,
        "remoteData": {
            "url": "https://remote.org/data.json",
            "format": "geojson",
            "dynamic": True,
        },
    }
    DataLayerFactory(map=map, settings=settings)
    data = [
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {"name": "Point 2"},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [4.3375, 11.2707],
                    },
                }
            ],
        },
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {"name": "Point 1"},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [4.3375, 12.2707],
                    },
                }
            ],
        },
    ]

    def handle(route):
        route.fulfill(json=data.pop())

    page.route("https://remote.org/data.json", handle)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#7/12.271/4.338")
    expect(page.locator(".umap-div-icon")).to_have_count(1)
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()
    page.get_by_role("button", name="Zoom out").click()

    # We are above fromZoom
    expect(page.locator(".umap-div-icon")).to_have_count(0)

    page.get_by_role("button", name="Zoom in").click()

    expect(page.locator(".umap-div-icon")).to_have_count(1)
    expect(page.get_by_role("tooltip", name="Point 2")).to_be_visible()
